#define GAME_DLL

//disabled client side
//for your own protection, because servers can still SendLua you
#define BLOCK_CLIENT_EXEC true

#include "common/GMLuaModule.h"
#include <cbase.h>

IVEngineServer* engine = NULL;
ICvar* cvar = NULL;
ILuaInterface* g_Lua = NULL;

GMOD_MODULE( Init, Shutdown );

LUA_FUNCTION( GetVal )
{

	g_Lua->CheckType( 1, GLua::TYPE_STRING );

	try
	{

		const char* Key = g_Lua->GetString( 1 );

		ConVar* NamedCvar = cvar->FindVar( Key );

		if( NamedCvar )
		{

			g_Lua->Push( NamedCvar->GetString( ) );

		}

	}
	catch( char* Exc )
	{

		g_Lua->Error( Exc );

		return 0;

	}

	return 1;

}

LUA_FUNCTION( SetVal )
{

	g_Lua->CheckType( 1, GLua::TYPE_STRING );
	g_Lua->CheckType( 2, GLua::TYPE_STRING );

	try
	{

		const char* Key = g_Lua->GetString( 1 );
		const char* Value = g_Lua->GetString( 2 );

		ConVar* NamedCvar = cvar->FindVar( Key );

		if ( NamedCvar )
		{

			NamedCvar->SetValue( Value );

			g_Lua->Push( true );

		}
		else
		{

			g_Lua->Push( false );

		}

	}
	catch( char* Exc )
	{

		g_Lua->Error( Exc );

		return 0;

	}

	return 1;

}

LUA_FUNCTION( Print )
{

	g_Lua->CheckType( 1, GLua::TYPE_TABLE );
	g_Lua->CheckType( 2, GLua::TYPE_STRING );

	try
	{

		ILuaObject* MsgCol = g_Lua->GetObject( 1 );
		const char* MsgStr = g_Lua->GetString( 2 );

		int R = MsgCol->GetMemberInt( "r" );
		int G = MsgCol->GetMemberInt( "g" );
		int B = MsgCol->GetMemberInt( "b" );

		Color Clr;
		Clr.SetColor( R, G, B, 255 );

		ConColorMsg( Clr, MsgStr );

	}
	catch( char* Exc )
	{

		g_Lua->Error( Exc );

		return 0;

	}

	return 1;

}

LUA_FUNCTION( LogPrint )
{

	g_Lua->CheckType( 1, GLua::TYPE_STRING );

	try
	{

		const char* MsgStr = g_Lua->GetString( 1 );

		engine->LogPrint( MsgStr );

		g_Lua->Msg( MsgStr );

	}
	catch(char* Exc)
	{

		g_Lua->Error( Exc );

		return 0;

	}

	return 1;

}

LUA_FUNCTION( RunCommand )
{

	g_Lua->CheckType( 1, GLua::TYPE_STRING );

	try
	{

		const char* Command = g_Lua->GetString( 1 );

		if( BLOCK_CLIENT_EXEC == true && g_Lua->IsClient( ) )
		{

			g_Lua->Msg( "Tried to execute string %q client side!\n", Command );

			return 0;

		}
		
		engine->ServerCommand( Command );

	}
	catch( char* Exc )
	{

		g_Lua->Error( Exc );

		return 0;

	}

	return 1;

}

int Init( lua_State* L )
{

	g_Lua = Lua( );

	if( g_Lua )
	{

		CreateInterfaceFn EngineFactory = Sys_GetFactory( "engine.dll" );
		CreateInterfaceFn ConVarFactory = Sys_GetFactory( "client.dll" );

		engine = ( IVEngineServer* )EngineFactory( INTERFACEVERSION_VENGINESERVER, NULL );

		//nothing i tried works here
		//is there a different method for getting and setting convars?
		//i know i can set them by passing a formatted string to the runcommand function but meh
		cvar = ( ICvar* )ConVarFactory( CVAR_INTERFACE_VERSION, NULL );

		if(engine)
		{

			ILuaObject* Table = g_Lua->GetNewTable( );

			if( cvar )
			{

				Table->SetMember( "GetValue", GetVal );
				Table->SetMember( "SetValue", SetVal );

			}
			else
			{

				//no point in spamming you with messages if we know it won't work
				//g_Lua->Msg( "Could not load convar interface! Ignoring..\n" );

			}

			Table->SetMember( "RunCommand", RunCommand );
			Table->SetMember( "Print", Print );
			Table->SetMember( "LogPrint", LogPrint );

			g_Lua->SetGlobal( "console", Table );

			return 1;

		}
		else
		{

			g_Lua->Error( "Could not load engine interface!\n" );

			return 0;

		}

	}

	return 0;

}

int Shutdown( lua_State* L )
{

	return 1;

}